@@ -50,80 +50,6 @@
 
 /* #define DEBUG 1 */
 
-int const CARTRIDGE_kb[CARTRIDGE_LAST_SUPPORTED + 1] = {
-	0,
-	8,        /* CARTRIDGE_STD_8 */
-	16,       /* CARTRIDGE_STD_16 */
-	16,       /* CARTRIDGE_OSS_034M_16 */
-	32,       /* CARTRIDGE_5200_32 */
-	32,       /* CARTRIDGE_DB_32 */
-	16,       /* CARTRIDGE_5200_EE_16 */
-	40,       /* CARTRIDGE_5200_40 */
-	64,       /* CARTRIDGE_WILL_64 */
-	64,       /* CARTRIDGE_EXP_64 */
-	64,       /* CARTRIDGE_DIAMOND_64 */
-	64,       /* CARTRIDGE_SDX_64 */
-	32,       /* CARTRIDGE_XEGS_32 */
-	64,       /* CARTRIDGE_XEGS_64_07 */
-	128,      /* CARTRIDGE_XEGS_128 */
-	16,       /* CARTRIDGE_OSS_M091_16 */
-	16,       /* CARTRIDGE_5200_NS_16 */
-	128,      /* CARTRIDGE_ATRAX_DEC_128 */
-	40,       /* CARTRIDGE_BBSB_40 */
-	8,        /* CARTRIDGE_5200_8 */
-	4,        /* CARTRIDGE_5200_4 */
-	8,        /* CARTRIDGE_RIGHT_8 */
-	32,       /* CARTRIDGE_WILL_32 */
-	256,      /* CARTRIDGE_XEGS_256 */
-	512,      /* CARTRIDGE_XEGS_512 */
-	1024,     /* CARTRIDGE_XEGS_1024 */
-	16,       /* CARTRIDGE_MEGA_16 */
-	32,       /* CARTRIDGE_MEGA_32 */
-	64,       /* CARTRIDGE_MEGA_64 */
-	128,      /* CARTRIDGE_MEGA_128 */
-	256,      /* CARTRIDGE_MEGA_256 */
-	512,      /* CARTRIDGE_MEGA_512 */
-	1024,     /* CARTRIDGE_MEGA_1024 */
-	32,       /* CARTRIDGE_SWXEGS_32 */
-	64,       /* CARTRIDGE_SWXEGS_64 */
-	128,      /* CARTRIDGE_SWXEGS_128 */
-	256,      /* CARTRIDGE_SWXEGS_256 */
-	512,      /* CARTRIDGE_SWXEGS_512 */
-	1024,     /* CARTRIDGE_SWXEGS_1024 */
-	8,        /* CARTRIDGE_PHOENIX_8 */
-	16,       /* CARTRIDGE_BLIZZARD_16 */
-	128,      /* CARTRIDGE_ATMAX_128 */
-	1024,     /* CARTRIDGE_ATMAX_1024 */
-	128,      /* CARTRIDGE_SDX_128 */
-	8,        /* CARTRIDGE_OSS_8 */
-	16,       /* CARTRIDGE_OSS_043M_16 */
-	4,        /* CARTRIDGE_BLIZZARD_4 */
-	32,       /* CARTRIDGE_AST_32 */
-	64,       /* CARTRIDGE_ATRAX_SDX_64 */
-	128,      /* CARTRIDGE_ATRAX_SDX_128 */
-	64,       /* CARTRIDGE_TURBOSOFT_64 */
-	128,      /* CARTRIDGE_TURBOSOFT_128 */
-	32,       /* CARTRIDGE_ULTRACART_32 */
-	8,        /* CARTRIDGE_LOW_BANK_8 */
-	128,      /* CARTRIDGE_SIC_128 */
-	256,      /* CARTRIDGE_SIC_256 */
-	512,      /* CARTRIDGE_SIC_512 */
-	2,        /* CARTRIDGE_STD_2 */
-	4,        /* CARTRIDGE_STD_4 */
-	4,        /* CARTRIDGE_RIGHT_4 */
-	32,       /* CARTRIDGE_TURBO_HIT_32 */
-	2048,     /* CARTRIDGE_MEGA_2048 */
-	128*1024, /* CARTRIDGE_THECART_128M */
-	4096,     /* CARTRIDGE_MEGA_4096 */
-	2048,     /* CARTRIDGE_MEGA_2048 */
-	32*1024,  /* CARTRIDGE_THECART_32M */
-	64*1024,  /* CARTRIDGE_THECART_64M */
-	64,       /* CARTRIDGE_XEGS_64_8F */
-	128,      /* CARTRIDGE_ATRAX_128 */
-	32,       /* CARTRIDGE_ADAWLIAH_32 */
-	64        /* CARTRIDGE_ADAWLIAH_64 */
-};
-
 int CARTRIDGE_autoreboot = TRUE;
 
 static int CartIsFor5200(int type)
@@ -135,6 +61,10 @@ static int CartIsFor5200(int type)
 	case CARTRIDGE_5200_NS_16:
 	case CARTRIDGE_5200_8:
 	case CARTRIDGE_5200_4:
+	case CARTRIDGE_5200_SUPER_64:
+	case CARTRIDGE_5200_SUPER_128:
+	case CARTRIDGE_5200_SUPER_256:
+	case CARTRIDGE_5200_SUPER_512:
 		return TRUE;
 	default:
 		break;
@@ -200,10 +130,10 @@ static void set_bank_A0AF(int main, int old_state)
 	}
 }
 
-/* WILL_64, EXP_64, DIAMOND_64, SDX_64, WILL_32, ATMAX_128, ATMAX_1024,
+/* WILL_64, EXP_64, DIAMOND_64, SDX_64, WILL_32, ATMAX_128, ATMAX_OLD_1024,
    ATRAX_DEC_128, ATRAX_SDX_64, TURBOSOFT_64, TURBOSOFT_128, ULTRACART_32,
    TURBO_HIT_32, THECART_128M, THECART_32M, THECART_64M, ATRAX_128, ADAWLIAH_32,
-   ADAWLIAH_64 */
+   ADAWLIAH_64, ATMAX_NEW_1024 */
 static void set_bank_A0BF(int disable_mask, int bank_mask)
 {
 	if (active_cart->state & disable_mask)
@@ -229,6 +159,11 @@ static void set_bank_80BF(void)
 	}
 }
 
+static void set_bank_5200_SUPER(void)
+{
+	MEMORY_CopyROM(0x4000, 0xbfff, active_cart->image + active_cart->state * 0x8000);
+}
+
 static void set_bank_SDX_128(void)
 {
 	if (active_cart->state & 8)
@@ -325,8 +260,9 @@ static void SwitchBank(int old_state)
 		set_bank_809F(0xfe000, old_state);
 		break;
 	case CARTRIDGE_ATRAX_DEC_128:
-	case CARTRIDGE_ATMAX_1024:
+	case CARTRIDGE_ATMAX_OLD_1024:
 	case CARTRIDGE_ATRAX_128:
+	case CARTRIDGE_ATMAX_NEW_1024:
 		set_bank_A0BF(0x80, 0x7f);
 		break;
 	case CARTRIDGE_ATMAX_128:
@@ -406,9 +342,22 @@ static void SwitchBank(int old_state)
 static void MapActiveCart(void)
 {
 	if (Atari800_machine_type == Atari800_MACHINE_5200) {
-		MEMORY_SetROM(0x4ff6, 0x4ff9);		/* disable Bounty Bob bank switching */
+		MEMORY_SetROM(0x4ff6, 0x4ff9); /* disable Bounty Bob bank switching */
 		MEMORY_SetROM(0x5ff6, 0x5ff9);
+		MEMORY_SetROM(0xbfc0, 0xbfff); /* disable Super Cart bank switching */
 		switch (active_cart->type) {
+		case CARTRIDGE_5200_SUPER_64:
+		case CARTRIDGE_5200_SUPER_128:
+		case CARTRIDGE_5200_SUPER_256:
+		case CARTRIDGE_5200_SUPER_512:
+			set_bank_5200_SUPER();
+#ifndef PAGED_ATTRIB
+			MEMORY_SetHARDWARE(0xbfc0, 0xbfff);
+#else
+			MEMORY_readmap[0xbf] = CARTRIDGE_5200SuperCartGetByte;
+			MEMORY_writemap[0xbf] = CARTRIDGE_5200SuperCartPutByte;
+#endif
+			break;
 		case CARTRIDGE_5200_32:
 			MEMORY_CopyROM(0x4000, 0xbfff, active_cart->image);
 			break;
@@ -511,7 +460,7 @@ static void MapActiveCart(void)
 		case CARTRIDGE_ATRAX_DEC_128:
 		case CARTRIDGE_WILL_32:
 		case CARTRIDGE_ATMAX_128:
-		case CARTRIDGE_ATMAX_1024:
+		case CARTRIDGE_ATMAX_OLD_1024:
 		case CARTRIDGE_SDX_128:
 		case CARTRIDGE_ATRAX_SDX_64:
 		case CARTRIDGE_ATRAX_SDX_128:
@@ -525,6 +474,7 @@ static void MapActiveCart(void)
 		case CARTRIDGE_ATRAX_128:
 		case CARTRIDGE_ADAWLIAH_32:
 		case CARTRIDGE_ADAWLIAH_64:
+		case CARTRIDGE_ATMAX_NEW_1024:
 			MEMORY_Cart809fDisable();
 			break;
 		case CARTRIDGE_DB_32:
@@ -856,8 +806,9 @@ static int access_D5(CARTRIDGE_image_t *cart, UWORD addr, int *state)
 	case CARTRIDGE_TURBOSOFT_64:
 		new_state = addr & 0x17;
 		break;
-	case CARTRIDGE_ATMAX_1024:
+	case CARTRIDGE_ATMAX_OLD_1024:
 	case CARTRIDGE_MEGAMAX_2048:
+	case CARTRIDGE_ATMAX_NEW_1024:
 		new_state = addr;
 		break;
 	case CARTRIDGE_OSS_8:
@@ -1106,114 +1057,103 @@ void CARTRIDGE_PutByte(UWORD addr, UBYTE byte)
 	PutByte(&CARTRIDGE_piggyback, addr, byte);
 }
 
-/* special support of Bounty Bob on Atari5200 */
-void CARTRIDGE_BountyBob1(UWORD addr)
+/* special support of Atari 5200 and Atari 8-bit Bounty Bob */
+
+/* addr must be $4fxx in 5200 mode or $8fxx in 800 mode. */
+static void access_BountyBob1(UWORD addr)
 {
-	if (Atari800_machine_type == Atari800_MACHINE_5200) {
-		if (addr >= 0x4ff6 && addr <= 0x4ff9) {
-			addr -= 0x4ff6;
-			MEMORY_CopyROM(0x4000, 0x4fff, active_cart->image + addr * 0x1000);
-			active_cart->state = (active_cart->state & 0x0c) | addr;
-		}
-	} else {
-		if (addr >= 0x8ff6 && addr <= 0x8ff9) {
-			addr -= 0x8ff6;
-			MEMORY_CopyROM(0x8000, 0x8fff, active_cart->image + addr * 0x1000);
-			active_cart->state = (active_cart->state & 0x0c) | addr;
+	UWORD base_addr = (addr & 0xf000);
+	addr &= 0x00ff;
+	if (addr >= 0xf6 && addr <= 0xf9) {
+		int new_state;
+		addr -= 0xf6;
+		new_state = (active_cart->state & 0x0c) | addr;
+		if (new_state != active_cart->state) {
+			MEMORY_CopyROM(base_addr, base_addr + 0x0fff,
+			               active_cart->image + addr * 0x1000);
+			active_cart->state = new_state;
 		}
 	}
 }
 
-void CARTRIDGE_BountyBob2(UWORD addr)
+/* addr must be $5fxx in 5200 mode or $9fxx in 800 mode. */
+static void access_BountyBob2(UWORD addr)
 {
-	if (Atari800_machine_type == Atari800_MACHINE_5200) {
-		if (addr >= 0x5ff6 && addr <= 0x5ff9) {
-			addr -= 0x5ff6;
-			MEMORY_CopyROM(0x5000, 0x5fff, active_cart->image + 0x4000 + addr * 0x1000);
-			active_cart->state = (active_cart->state & 0x03) | (addr << 2);
-		}
-	}
-	else {
-		if (addr >= 0x9ff6 && addr <= 0x9ff9) {
-			addr -= 0x9ff6;
-			MEMORY_CopyROM(0x9000, 0x9fff, active_cart->image + 0x4000 + addr * 0x1000);
-			active_cart->state = (active_cart->state & 0x03) | (addr << 2);
+	UWORD base_addr = (addr & 0xf000);
+	addr &= 0x00ff;
+	if (addr >= 0xf6 && addr <= 0xf9) {
+		int new_state;
+		addr -= 0xf6;
+		new_state = (active_cart->state & 0x03) | (addr << 2);
+		if (new_state != active_cart->state) {
+			MEMORY_CopyROM(base_addr, base_addr + 0x0fff,
+			               active_cart->image + 0x4000 + addr * 0x1000);
+			active_cart->state = new_state;
 		}
 	}
 }
 
-#ifdef PAGED_ATTRIB
+/* addr must be $bfxx in 5200 mode only. */
+static void access_5200SuperCart(UWORD addr)
+{
+	int old_state = active_cart->state;
+	int new_state = old_state;
+
+	if ((addr & 0xc0) == 0xc0) {
+		switch (addr & 0x30) {
+		case 0x00: /* $BFCx */
+			new_state = (new_state & 0x03) | (addr & 0x0c);
+			break;
+		case 0x10: /* $BFDx */
+			new_state = (new_state & 0x0c) | ((addr & 0x0c) >> 2);
+			break;
+		default: /* 0x20 or 0x30, i.e. $BFEx or $BFFx */
+			new_state = 0x0f;
+			break;
+		}
+		new_state &= ((active_cart->size >> 5) - 1);
+	}
+
+	if (old_state != new_state) {
+		active_cart->state = new_state;
+		set_bank_5200_SUPER();
+	}
+}
+
 UBYTE CARTRIDGE_BountyBob1GetByte(UWORD addr, int no_side_effects)
 {
-	if (!no_side_effects) {
-		if (Atari800_machine_type == Atari800_MACHINE_5200) {
-			if (addr >= 0x4ff6 && addr <= 0x4ff9) {
-				CARTRIDGE_BountyBob1(addr);
-				return 0;
-			}
-		} else {
-			if (addr >= 0x8ff6 && addr <= 0x8ff9) {
-				CARTRIDGE_BountyBob1(addr);
-				return 0;
-			}
-		}
-	}
+	if (!no_side_effects)
+		access_BountyBob1(addr);
 	return MEMORY_dGetByte(addr);
 }
 
 UBYTE CARTRIDGE_BountyBob2GetByte(UWORD addr, int no_side_effects)
 {
-	if (!no_side_effects) {
-		if (Atari800_machine_type == Atari800_MACHINE_5200) {
-			if (addr >= 0x5ff6 && addr <= 0x5ff9) {
-				CARTRIDGE_BountyBob2(addr);
-				return 0;
-			}
-		} else {
-			if (addr >= 0x9ff6 && addr <= 0x9ff9) {
-				CARTRIDGE_BountyBob2(addr);
-				return 0;
-			}
-		}
-	}
+	if (!no_side_effects)
+		access_BountyBob2(addr);
+	return MEMORY_dGetByte(addr);
+}
+
+UBYTE CARTRIDGE_5200SuperCartGetByte(UWORD addr, int no_side_effects)
+{
+	if (!no_side_effects)
+		access_5200SuperCart(addr);
 	return MEMORY_dGetByte(addr);
 }
 
 void CARTRIDGE_BountyBob1PutByte(UWORD addr, UBYTE value)
 {
-	if (Atari800_machine_type == Atari800_MACHINE_5200) {
-		if (addr >= 0x4ff6 && addr <= 0x4ff9) {
-			CARTRIDGE_BountyBob1(addr);
-		}
-	} else {
-		if (addr >= 0x8ff6 && addr <= 0x8ff9) {
-			CARTRIDGE_BountyBob1(addr);
-		}
-	}
+	access_BountyBob1(addr);
 }
 
 void CARTRIDGE_BountyBob2PutByte(UWORD addr, UBYTE value)
 {
-	if (Atari800_machine_type == Atari800_MACHINE_5200) {
-		if (addr >= 0x5ff6 && addr <= 0x5ff9) {
-			CARTRIDGE_BountyBob2(addr);
-		}
-	} else {
-		if (addr >= 0x9ff6 && addr <= 0x9ff9) {
-			CARTRIDGE_BountyBob2(addr);
-		}
-	}
+	access_BountyBob2(addr);
 }
-#endif
 
-int CARTRIDGE_Checksum(const UBYTE *image, int nbytes)
+void CARTRIDGE_5200SuperCartPutByte(UWORD addr, UBYTE value)
 {
-	int checksum = 0;
-	while (nbytes > 0) {
-		checksum += *image++;
-		nbytes--;
-	}
-	return checksum;
+	access_5200SuperCart(addr);
 }
 
 static void ResetCartState(CARTRIDGE_image_t *cart)
@@ -1222,7 +1162,7 @@ static void ResetCartState(CARTRIDGE_image_t *cart)
 	case CARTRIDGE_OSS_034M_16:
 		cart->state = 1;
 		break;
-	case CARTRIDGE_ATMAX_1024:
+	case CARTRIDGE_ATMAX_OLD_1024:
 		cart->state = 0x7f;
 		break;
 	case CARTRIDGE_AST_32:
@@ -1435,9 +1375,20 @@ void CARTRIDGE_ColdStart(void) {
 	MapActiveCart();
 }
 
-/* Loads a cartridge from FILENAME. Copies FILENAME to CART_FILENAME.
-   Allocates a buffer with cartridge image data and puts it in *CART_IMAGE.
-   Sets *CART_TYPE to the cartridge type. */
+/* Loads a cartridge from FILENAME. Copies FILENAME to CART->FILENAME.
+   If loading failed, sets CART->TYPE to CARTRIDGE_NONE and returns one of:
+   * CARTRIDGE_CANT_OPEN if there was an error when opening file,
+   * CARTRIDGE_BAD_FORMAT if the file is not a proper cartridge image.
+
+   If loading succeeded, allocates a buffer with cartridge image data and puts
+   it in CART->IMAGE. Then sets CART->TYPE if possible, and returns one of:
+   * 0 if cartridge type was recognized; CART->TYPE is then set correctly;
+   * CARTRIDGE_BAD_CHECKSUM if cartridge is a CART file but with invalid
+     checksum; CART->TYPE is then set correctly;
+   * a positive integer: size in KB if cartridge type was not guessed;
+     CART->TYPE is then set to CARTRIDGE_UNKNOWN. The caller is expected to
+     select a cartridge type according to the returned size, and call either
+     CARTRIDGE_SetType() or CARTRIDGE_SetTypeAutoReboot(). */
 static int InsertCartridge(const char *filename, CARTRIDGE_image_t *cart)
 {
 	FILE *fp;
@@ -1470,8 +1421,8 @@ static int InsertCartridge(const char *filename, CARTRIDGE_image_t *cart)
 		cart->type = CARTRIDGE_NONE;
 		len >>= 10;	/* number of kilobytes */
 		cart->size = len;
-		for (type = 1; type <= CARTRIDGE_LAST_SUPPORTED; type++)
-			if (CARTRIDGE_kb[type] == len) {
+		for (type = 1; type < CARTRIDGE_TYPE_COUNT; type++)
+			if (CARTRIDGES[type].kb == len) {
 				if (cart->type == CARTRIDGE_NONE) {
 					cart->type = type;
 				} else {
@@ -1500,11 +1451,11 @@ static int InsertCartridge(const char *filename, CARTRIDGE_image_t *cart)
 			(header[5] << 16) |
 			(header[6] << 8) |
 			header[7];
-		if (type >= 1 && type <= CARTRIDGE_LAST_SUPPORTED) {
+		if (type >= 1 && type < CARTRIDGE_TYPE_COUNT) {
 			int checksum;
 			int result;
-			len = CARTRIDGE_kb[type] << 10;
-			cart->size = CARTRIDGE_kb[type];
+			len = CARTRIDGES[type].kb << 10;
+			cart->size = CARTRIDGES[type].kb;
 			/* alloc memory and read data */
 			cart->image = (UBYTE *) Util_malloc(len);
 			if (fread(cart->image, 1, len, fp) < len) {
@@ -1573,7 +1524,7 @@ int CARTRIDGE_ReadConfig(char *string, char *ptr)
 	}
 	else if (strcmp(string, "CARTRIDGE_TYPE") == 0) {
 		int value = Util_sscandec(ptr);
-		if (value < 0 || value > CARTRIDGE_LAST_SUPPORTED)
+		if (value < 0 || value >= CARTRIDGE_TYPE_COUNT)
 			return FALSE;
 		CARTRIDGE_main.type = value;
 	}
@@ -1584,7 +1535,7 @@ int CARTRIDGE_ReadConfig(char *string, char *ptr)
 	}
 	else if (strcmp(string, "CARTRIDGE_PIGGYBACK_TYPE") == 0) {
 		int value = Util_sscandec(ptr);
-		if (value < 0 || value > CARTRIDGE_LAST_SUPPORTED)
+		if (value < 0 || value >= CARTRIDGE_TYPE_COUNT)
 			return FALSE;
 		CARTRIDGE_piggyback.type = value;
 	}
@@ -1619,7 +1570,7 @@ static void InitInsert(CARTRIDGE_image_t *cart)
 			/* Assume r == CARTRIDGE_BAD_CHECKSUM */ "Bad checksum");
 			cart->type = CARTRIDGE_NONE;
 		}
-		if (cart->type == CARTRIDGE_UNKNOWN && CARTRIDGE_kb[tmp_type] == res)
+		if (cart->type == CARTRIDGE_UNKNOWN && CARTRIDGES[tmp_type].kb == res)
 			CARTRIDGE_SetType(cart, tmp_type);
 	}
 }
@@ -1652,7 +1603,7 @@ int CARTRIDGE_Initialise(int *argc, char *argv[])
 		else if (strcmp(argv[i], "-cart-type") == 0) {
 			if (i_a) {
 				Util_sscansdec(argv[++i], &CARTRIDGE_main.type);
-				if (CARTRIDGE_main.type < 0 ||  CARTRIDGE_main.type > CARTRIDGE_LAST_SUPPORTED)
+				if (CARTRIDGE_main.type < 0 ||  CARTRIDGE_main.type >= CARTRIDGE_TYPE_COUNT)
 					a_i = TRUE;
 				else
 					type_from_commandline = TRUE;
@@ -1670,7 +1621,7 @@ int CARTRIDGE_Initialise(int *argc, char *argv[])
 		else if (strcmp(argv[i], "-cart2-type") == 0) {
 			if (i_a) {
 				Util_sscansdec(argv[++i], &CARTRIDGE_piggyback.type);
-				if (CARTRIDGE_piggyback.type < 0 ||  CARTRIDGE_piggyback.type > CARTRIDGE_LAST_SUPPORTED)
+				if (CARTRIDGE_piggyback.type < 0 ||  CARTRIDGE_piggyback.type >= CARTRIDGE_TYPE_COUNT)
 					a_i = TRUE;
 				else
 					type2_from_commandline = TRUE;
@@ -1685,9 +1636,9 @@ int CARTRIDGE_Initialise(int *argc, char *argv[])
 			if (strcmp(argv[i], "-help") == 0) {
 				help_only = TRUE;
 				Log_print("\t-cart <file>         Install cartridge (raw or CART format)");
-				Log_print("\t-cart-type <num>     Set cartridge type (0..%i)", CARTRIDGE_LAST_SUPPORTED);
+				Log_print("\t-cart-type <num>     Set cartridge type (0..%i)", CARTRIDGE_TYPE_COUNT-1);
 				Log_print("\t-cart2 <file>        Install piggyback cartridge");
-				Log_print("\t-cart2-type <num>    Set piggyback cartridge type (0..%i)", CARTRIDGE_LAST_SUPPORTED);
+				Log_print("\t-cart2-type <num>    Set piggyback cartridge type (0..%i)", CARTRIDGE_TYPE_COUNT-1);
 				Log_print("\t-cart-autoreboot     Reboot when cartridge is inserted/removed");
 				Log_print("\t-no-cart-autoreboot  Don't reboot after changing cartridge");
 			}
