@@ -62,6 +62,9 @@
 #ifdef SOUND
 #include "sound.h"
 #endif
+#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+#include "file_export.h"
+#endif
 
 int CFG_save_on_exit = FALSE;
 
@@ -126,6 +129,11 @@ int CFG_LoadConfig(const char *alternate_config_filename)
 	while (fgets(string, sizeof(string), fp)) {
 		char *ptr;
 		Util_chomp(string);
+		Util_trim(string);
+		/* Check for comments */
+		if (string[0] == '#') {
+			continue;
+		}
 		ptr = strchr(string, '=');
 		if (ptr != NULL) {
 			*ptr++ = '\0';
@@ -330,6 +338,10 @@ int CFG_LoadConfig(const char *alternate_config_filename)
 			else if (Sound_ReadConfig(string, ptr)) {
 			}
 #endif /* defined(SOUND) && defined(SOUND_THIN_API) */
+#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+			else if (File_Export_ReadConfig(string, ptr)) {
+			}
+#endif
 			else {
 #ifdef SUPPORTS_PLATFORM_CONFIGURE
 				if (!PLATFORM_Configure(string, ptr)) {
@@ -471,6 +483,9 @@ int CFG_WriteConfig(void)
 #if defined(SOUND) && defined(SOUND_THIN_API)
 	Sound_WriteConfig(fp);
 #endif /* defined(SOUND) && defined(SOUND_THIN_API) */
+#if (defined(SOUND) && !defined(DREAMCAST)) || defined(VIDEO_RECORDING)
+	File_Export_WriteConfig(fp);
+#endif
 #ifdef SUPPORTS_PLATFORM_CONFIGSAVE
 	PLATFORM_ConfigSave(fp);
 #endif
