@@ -8,9 +8,10 @@ extern int PBI_BB_enabled;
 void PBI_BB_Menu(void);
 void PBI_BB_Frame(void);
 int PBI_BB_Initialise(int *argc, char *argv[]);
-UBYTE PBI_BB_D1GetByte(UWORD addr);
+void PBI_BB_Exit(void);
+UBYTE PBI_BB_D1GetByte(UWORD addr, int no_side_effects);
 void PBI_BB_D1PutByte(UWORD addr, UBYTE byte);
-UBYTE PBI_BB_D6GetByte(UWORD addr);
+UBYTE PBI_BB_D6GetByte(UWORD addr, int no_side_effects);
 void PBI_BB_D6PutByte(UWORD addr, UBYTE byte);
 int PBI_BB_ReadConfig(char *string, char *ptr);
 void PBI_BB_WriteConfig(FILE *fp);
