@@ -78,6 +78,19 @@ static int KBD_STICK_1_RIGHT = SDLK_d;
 static int KBD_STICK_1_DOWN = SDLK_s;
 static int KBD_STICK_1_UP = SDLK_w;
 
+/* Maping for the START, RESET, OPTION, SELECT and EXIT keys */
+static int KBD_UI = SDLK_F1;
+static int KBD_OPTION = SDLK_F2;
+static int KBD_SELECT = SDLK_F3;
+static int KBD_START = SDLK_F4;
+static int KBD_RESET = SDLK_F5;
+static int KBD_HELP = SDLK_F6;
+static int KBD_BREAK = SDLK_F7;
+static int KBD_MON = SDLK_F8;
+static int KBD_EXIT = SDLK_F9;
+static int KBD_SSHOT = SDLK_F10;
+static int KBD_TURBO = SDLK_F12;
+
 /* real joysticks */
 
 static int fd_joystick0 = -1;
@@ -223,6 +236,28 @@ int SDL_INPUT_ReadConfig(char *option, char *parameters)
 		return set_real_js_use_hat(2,parameters);
 	else if (strcmp(option, "SDL_JOY_3_USE_HAT") == 0)
 		return set_real_js_use_hat(3,parameters);
+	else if (strcmp(option, "SDL_UI_KEY") == 0)
+		return SDLKeyBind(&KBD_UI, parameters);
+	else if (strcmp(option, "SDL_OPTION_KEY") == 0)
+		return SDLKeyBind(&KBD_OPTION, parameters);
+	else if (strcmp(option, "SDL_SELECT_KEY") == 0)
+		return SDLKeyBind(&KBD_SELECT, parameters);
+	else if (strcmp(option, "SDL_START_KEY") == 0)
+		return SDLKeyBind(&KBD_START, parameters);
+	else if (strcmp(option, "SDL_RESET_KEY") == 0)
+		return SDLKeyBind(&KBD_RESET, parameters);
+	else if (strcmp(option, "SDL_HELP_KEY") == 0)
+		return SDLKeyBind(&KBD_HELP, parameters);
+	else if (strcmp(option, "SDL_BREAK_KEY") == 0)
+		return SDLKeyBind(&KBD_BREAK, parameters);
+	else if (strcmp(option, "SDL_MON_KEY") == 0)
+		return SDLKeyBind(&KBD_MON, parameters);
+	else if (strcmp(option, "SDL_EXIT_KEY") == 0)
+		return SDLKeyBind(&KBD_EXIT, parameters);
+	else if (strcmp(option, "SDL_SSHOT_KEY") == 0)
+		return SDLKeyBind(&KBD_SSHOT, parameters);
+	else if (strcmp(option, "SDL_TURBO_KEY") == 0)
+		return SDLKeyBind(&KBD_TURBO, parameters);
 	else
 		return FALSE;
 }
@@ -246,6 +281,18 @@ void SDL_INPUT_WriteConfig(FILE *fp)
 	fprintf(fp, "SDL_JOY_1_DOWN=%d\n", KBD_STICK_1_DOWN);
 	fprintf(fp, "SDL_JOY_1_TRIGGER=%d\n", KBD_TRIG_1);
 
+	fprintf(fp, "SDL_UI_KEY=%d\n", KBD_UI);
+	fprintf(fp, "SDL_OPTION_KEY=%d\n", KBD_OPTION);
+	fprintf(fp, "SDL_SELECT_KEY=%d\n", KBD_SELECT);
+	fprintf(fp, "SDL_START_KEY=%d\n", KBD_START);
+	fprintf(fp, "SDL_RESET_KEY=%d\n", KBD_RESET);
+	fprintf(fp, "SDL_HELP_KEY=%d\n", KBD_HELP);
+	fprintf(fp, "SDL_BREAK_KEY=%d\n", KBD_BREAK);
+	fprintf(fp, "SDL_MON_KEY=%d\n", KBD_MON);
+	fprintf(fp, "SDL_EXIT_KEY=%d\n", KBD_EXIT);
+	fprintf(fp, "SDL_SSHOT_KEY=%d\n", KBD_SSHOT);
+	fprintf(fp, "SDL_TURBO_KEY=%d\n", KBD_TURBO);
+	
 	write_real_js_configs(fp);
 }
 
@@ -482,6 +529,9 @@ int PLATFORM_Keyboard(void)
 			case SDLK_w:
 				UI_alt_function = UI_MENU_SOUND_RECORDING;
 				break;
+			case SDLK_v:
+				UI_alt_function = UI_MENU_VIDEO_RECORDING;
+				break;
 			case SDLK_a:
 				UI_alt_function = UI_MENU_ABOUT;
 				break;
@@ -670,37 +720,53 @@ int PLATFORM_Keyboard(void)
 
 	/* OPTION / SELECT / START keys */
 	INPUT_key_consol = INPUT_CONSOL_NONE;
-	if (kbhits[SDLK_F2])
+	if (kbhits[KBD_OPTION])
 		INPUT_key_consol &= ~INPUT_CONSOL_OPTION;
-	if (kbhits[SDLK_F3])
+	if (kbhits[KBD_SELECT])
 		INPUT_key_consol &= ~INPUT_CONSOL_SELECT;
-	if (kbhits[SDLK_F4])
+	if (kbhits[KBD_START])
 		INPUT_key_consol &= ~INPUT_CONSOL_START;
 
 	if (key_pressed == 0)
 		return AKEY_NONE;
 
 	/* Handle movement and special keys. */
-	switch (lastkey) {
-	case SDLK_F1:
-		key_pressed = 0;
-		return AKEY_UI;
-	case SDLK_F5:
+
+	/* Since KBD_RESET & KBD_EXIT are variables, they can't be cases */
+	/* in a switch statement.  So handle them here with if-blocks */
+	if (lastkey == KBD_RESET) {
 		key_pressed = 0;
 		return INPUT_key_shift ? AKEY_COLDSTART : AKEY_WARMSTART;
-	case SDLK_F8:
-		UI_alt_function = UI_MENU_MONITOR;
-		break;
-	case SDLK_F9:
+	}
+	if (lastkey == KBD_EXIT) {
 		return AKEY_EXIT;
-	case SDLK_F10:
+	}
+	if (lastkey == KBD_UI) {
+		key_pressed = 0;
+		return AKEY_UI;
+	}
+	if (lastkey == KBD_MON) {
+		UI_alt_function = UI_MENU_MONITOR;
+	}
+	if (lastkey == KBD_HELP) {
+		return AKEY_HELP ^ shiftctrl;
+	}
+	if (lastkey == KBD_BREAK) {
+		if (BINLOAD_wait_active) {
+			BINLOAD_pause_loading = TRUE;
+			return AKEY_NONE;
+		}
+		else
+			return AKEY_BREAK;
+	}
+	if (lastkey == KBD_SSHOT) {
 		key_pressed = 0;
 		return INPUT_key_shift ? AKEY_SCREENSHOT_INTERLACE : AKEY_SCREENSHOT;
-	case SDLK_F12:
+	}
+	if (lastkey == KBD_TURBO) {
 		key_pressed = 0;
 		return AKEY_TURBO;
 	}
-
 	if (UI_alt_function != -1) {
 		key_pressed = 0;
 		return AKEY_UI;
@@ -779,8 +845,6 @@ int PLATFORM_Keyboard(void)
 		else
 			return AKEY_CAPSTOGGLE;
 	case SDLK_END:
-	case SDLK_F6:
-		return AKEY_HELP ^ shiftctrl;
 	case SDLK_PAGEDOWN:
 		return AKEY_F2 | AKEY_SHFT;
 	case SDLK_PAGEUP:
@@ -788,13 +852,6 @@ int PLATFORM_Keyboard(void)
 	case SDLK_HOME:
 		return key_control ? AKEY_LESS|shiftctrl : AKEY_CLEAR;
 	case SDLK_PAUSE:
-	case SDLK_F7:
-		if (BINLOAD_wait_active) {
-			BINLOAD_pause_loading = TRUE;
-			return AKEY_NONE;
-		}
-		else
-			return AKEY_BREAK;
 	case SDLK_CAPSLOCK:
 		if (INPUT_key_shift)
 			return AKEY_CAPSLOCK|shiftctrl;
@@ -1299,10 +1356,10 @@ int SDL_INPUT_Initialise(int *argc, char *argv[])
 			if (strcmp(argv[i], "-help") == 0) {
 				help_only = TRUE;
 				Log_print("\t-nojoystick      Disable joystick");
-                                Log_print("\t-joy0hat         Use hat of joystick 0");
-                                Log_print("\t-joy1hat         Use hat of joystick 1");
-                                Log_print("\t-joy2hat         Use hat of joystick 1");
-                                Log_print("\t-joy3hat         Use hat of joystick 1");
+				Log_print("\t-joy0hat         Use hat of joystick 0");
+				Log_print("\t-joy1hat         Use hat of joystick 1");
+				Log_print("\t-joy2hat         Use hat of joystick 2");
+				Log_print("\t-joy3hat         Use hat of joystick 3");
 #ifdef LPTJOY
 				Log_print("\t-joy0 <pathname> Select LPTjoy0 device");
 				Log_print("\t-joy1 <pathname> Select LPTjoy1 device");
