@@ -27,6 +27,7 @@
 #include <SDL.h>
 
 #include "af80.h"
+#include "bit3.h"
 #include "artifact.h"
 #include "atari.h"
 #include "colours.h"
@@ -55,21 +56,43 @@ int SDL_VIDEO_SW_bpp = 0;
 static void DisplayWithoutScaling(void);
 static void DisplayWithScaling(void);
 static void DisplayRotated(void);
+#ifdef NTSC_FILTER
 static void DisplayNTSCEmu(void);
+#endif
+#ifdef XEP80_EMULATION
 static void DisplayXEP80(void);
+#endif
+#ifdef PBI_PROTO80
 static void DisplayProto80(void);
+#endif
+#ifdef AF80
 static void DisplayAF80(void);
+#endif
+#ifdef BIT3
+static void DisplayBIT3(void);
+#endif
 #ifdef PAL_BLENDING
 static void DisplayPalBlending(void);
 static void DisplayPalBlendingScaled(void);
 #endif /* PAL_BLENDING */
 
 static void (*blit_funcs[VIDEOMODE_MODE_SIZE])(void) = {
-	&DisplayWithoutScaling,
-	&DisplayNTSCEmu,
-	&DisplayXEP80,
-	&DisplayProto80,
-	&DisplayAF80
+	&DisplayWithoutScaling
+#ifdef NTSC_FILTER
+	,&DisplayNTSCEmu
+#endif
+#ifdef XEP80_EMULATION
+	,&DisplayXEP80
+#endif
+#ifdef PBI_PROTO80
+	,&DisplayProto80
+#endif
+#ifdef AF80
+	,&DisplayAF80
+#endif
+#ifdef BIT3
+	,&DisplayBIT3
+#endif
 };
 
 static void Set8BitPalette(VIDEOMODE_MODE_t mode)
@@ -134,8 +157,8 @@ void SDL_VIDEO_SW_PaletteUpdate(void)
 
 static void ModeInfo(void)
 {
-	char *fullstring = fullscreen ? "fullscreen" : "windowed";
-	char *vsyncstring = (SDL_VIDEO_screen->flags & SDL_DOUBLEBUF) ? "with vsync" : "without vsync";
+	const char *fullstring = fullscreen ? "fullscreen" : "windowed";
+	const char *vsyncstring = (SDL_VIDEO_screen->flags & SDL_DOUBLEBUF) ? "with vsync" : "without vsync";
 	Log_print("Video Mode: %dx%dx%d %s %s", SDL_VIDEO_screen->w, SDL_VIDEO_screen->h,
 	          SDL_VIDEO_screen->format->BitsPerPixel, fullstring, vsyncstring);
 }
@@ -181,7 +204,10 @@ void SDL_VIDEO_SW_SetVideoMode(VIDEOMODE_resolution_t const *res, int windowed,
 	}
 
 	if ((rotate90 && SDL_VIDEO_SW_bpp != 16) ||
-		((mode == VIDEOMODE_MODE_NTSC_FILTER
+		((0
+#ifdef NTSC_FILTER
+		  || mode == VIDEOMODE_MODE_NTSC_FILTER
+#endif
 #ifdef PAL_BLENDING
 		  || (mode == VIDEOMODE_MODE_NORMAL && ARTIFACT_mode == ARTIFACT_PAL_BLEND)
 #endif /* PAL_BLENDING */
@@ -441,6 +467,7 @@ static void scanLines_32(void* pBuffer, int width, int height, int pitch, int sc
 	}
 }
 
+#ifdef XEP80_EMULATION
 static void DisplayXEP80(void)
 {
 	static int xep80Frame = 0;
@@ -473,7 +500,9 @@ static void DisplayXEP80(void)
 		scanLines_32((void *)pixels, VIDEOMODE_dest_width, VIDEOMODE_dest_height, SDL_VIDEO_screen->pitch, SDL_VIDEO_scanlines_percentage);
 	}
 }
+#endif
 
+#ifdef NTSC_FILTER
 static void DisplayNTSCEmu(void)
 {
 	Uint8 *pixels = (Uint8*)SDL_VIDEO_screen->pixels + SDL_VIDEO_screen->pitch * VIDEOMODE_dest_offset_top;
@@ -503,7 +532,9 @@ static void DisplayNTSCEmu(void)
 		break;
 	}
 }
+#endif
 
+#ifdef PBI_PROTO80
 static void DisplayProto80(void)
 {
 	int first_column = (VIDEOMODE_src_offset_left+7) / 8;
@@ -530,7 +561,9 @@ static void DisplayProto80(void)
 		scanLines_32((void *)pixels, VIDEOMODE_dest_width, VIDEOMODE_dest_height, SDL_VIDEO_screen->pitch, SDL_VIDEO_scanlines_percentage);
 	}
 }
+#endif
 
+#ifdef AF80
 static void DisplayAF80(void)
 {
 	int first_column = (VIDEOMODE_src_offset_left+7) / 8;
@@ -562,6 +595,41 @@ static void DisplayAF80(void)
 		scanLines_32((void *)pixels, VIDEOMODE_dest_width, VIDEOMODE_dest_height, SDL_VIDEO_screen->pitch, SDL_VIDEO_scanlines_percentage);
 	}
 }
+#endif
+
+#ifdef BIT3
+static void DisplayBIT3(void)
+{
+	int first_column = (VIDEOMODE_src_offset_left+7) / 8;
+	int last_column = (VIDEOMODE_src_offset_left + VIDEOMODE_src_width) / 8;
+	int first_line = VIDEOMODE_src_offset_top;
+	int last_line = first_line + VIDEOMODE_src_height;
+	int pitch4 = SDL_VIDEO_screen->pitch / 2;
+	Uint8 *pixels = (Uint8*)SDL_VIDEO_screen->pixels + SDL_VIDEO_screen->pitch * VIDEOMODE_dest_offset_top;
+
+	static int BIT3Frame = 0;
+	int blink;
+	BIT3Frame++;
+	if (BIT3Frame == 60) BIT3Frame = 0;
+	blink = BIT3Frame >= 30;
+	
+	switch (SDL_VIDEO_screen->format->BitsPerPixel) {
+	case 8:
+		pixels += VIDEOMODE_dest_offset_left;
+		SDL_VIDEO_BlitBIT3_8((Uint32 *)pixels, first_column, last_column, pitch4, first_line, last_line, blink);
+		break;
+	case 16:
+		pixels += VIDEOMODE_dest_offset_left * 2;
+		SDL_VIDEO_BlitBIT3_16((Uint32 *)pixels, first_column, last_column, pitch4, first_line, last_line, blink, SDL_PALETTE_buffer.bpp16);
+		scanLines_16((void *)pixels, VIDEOMODE_dest_width, VIDEOMODE_dest_height, SDL_VIDEO_screen->pitch, SDL_VIDEO_scanlines_percentage);
+		break;
+	default:
+		pixels += VIDEOMODE_dest_offset_left * 4;
+		SDL_VIDEO_BlitBIT3_32((Uint32 *)pixels, first_column, last_column, pitch4, first_line, last_line, blink, SDL_PALETTE_buffer.bpp32);
+		scanLines_32((void *)pixels, VIDEOMODE_dest_width, VIDEOMODE_dest_height, SDL_VIDEO_screen->pitch, SDL_VIDEO_scanlines_percentage);
+	}
+}
+#endif
 
 static void DisplayRotated(void)
 {
@@ -638,6 +706,16 @@ static void DisplayWithScaling(void)
 			pos = w1;
 			yy = Screen_WIDTH * (y >> 16);
 			while (pos >= 0) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				quad = (screen[yy + (x >> 16)] << 0);
+				x -= dx;
+				quad += (screen[yy + (x >> 16)] << 8);
+				x -= dx;
+				quad += (screen[yy + (x >> 16)] << 16);
+				x -= dx;
+				quad += (screen[yy + (x >> 16)] << 24);
+				x -= dx;
+#else
 				quad = (screen[yy + (x >> 16)] << 24);
 				x -= dx;
 				quad += (screen[yy + (x >> 16)] << 16);
@@ -646,6 +724,7 @@ static void DisplayWithScaling(void)
 				x -= dx;
 				quad += (screen[yy + (x >> 16)] << 0);
 				x -= dx;
+#endif
 
 				pixels[pos] = quad;
 				pos--;
@@ -664,12 +743,22 @@ static void DisplayWithScaling(void)
 			pos = w1;
 			yy = Screen_WIDTH * (y >> 16);
 			while (pos >= 0) {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+				c = screen[yy + (x >> 16)];
+				quad = SDL_PALETTE_buffer.bpp16[c];
+				x -= dx;
+				c = screen[yy + (x >> 16)];
+				quad += SDL_PALETTE_buffer.bpp16[c] << 16;
+				x -= dx;
+#else
 				c = screen[yy + (x >> 16)];
 				quad = SDL_PALETTE_buffer.bpp16[c] << 16;
 				x -= dx;
 				c = screen[yy + (x >> 16)];
 				quad += SDL_PALETTE_buffer.bpp16[c];
 				x -= dx;
+#endif
+
 				pixels[pos] = quad;
 				pos--;
 			}
@@ -711,11 +800,11 @@ static void DisplayPalBlending(void)
 	 * PLATFORM_SetVideoMode() function. */
 	case 16:
 		pixels += VIDEOMODE_dest_offset_left * 2;
-		PAL_BLENDING_Blit16((Uint32*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_src_offset_top % 2);
+		PAL_BLENDING_Blit16((ULONG*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_src_offset_top % 2);
 		break;
 	default: /* SDL_VIDEO_screen->format->BitsPerPixel == 32 */
 		pixels += VIDEOMODE_dest_offset_left * 4;
-		PAL_BLENDING_Blit32((Uint32 *)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_src_offset_top % 2);
+		PAL_BLENDING_Blit32((ULONG *)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_src_offset_top % 2);
 	}
 }
 
@@ -729,11 +818,11 @@ static void DisplayPalBlendingScaled(void)
 	 * PLATFORM_SetVideoMode() function. */
 	case 16:
 		pixels += pitch4 * VIDEOMODE_dest_offset_top + VIDEOMODE_dest_offset_left / 2;
-		PAL_BLENDING_BlitScaled16((Uint32*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_dest_width, VIDEOMODE_dest_height, VIDEOMODE_src_offset_top % 2);
+		PAL_BLENDING_BlitScaled16((ULONG*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_dest_width, VIDEOMODE_dest_height, VIDEOMODE_src_offset_top % 2);
 		break;
 	case 32:
 		pixels += pitch4 * VIDEOMODE_dest_offset_top + VIDEOMODE_dest_offset_left;
-		PAL_BLENDING_BlitScaled32((Uint32*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_dest_width, VIDEOMODE_dest_height, VIDEOMODE_src_offset_top % 2);
+		PAL_BLENDING_BlitScaled32((ULONG*)pixels, screen, pitch4, VIDEOMODE_src_width, VIDEOMODE_src_height, VIDEOMODE_dest_width, VIDEOMODE_dest_height, VIDEOMODE_src_offset_top % 2);
 	}
 }
 #endif /* PAL_BLENDING */
