@@ -50,10 +50,17 @@
 #  include <time.h>
 # endif
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h> /* getcwd() */
+#endif
+#ifdef HAVE_DIRECT_H
+#include <direct.h> /* getcwd on MSVC*/
+#endif
 
 #include "atari.h"
 #include "platform.h"
 #include "util.h"
+#include "log.h"
 
 int Util_chrieq(char c1, char c2)
 {
@@ -81,6 +88,15 @@ int Util_stricmp(const char *str1, const char *str2)
 }
 #endif
 
+int Util_striendswith(const char *s1, const char *s2)
+{
+	int pos;
+	pos = strlen(s1) - strlen(s2);
+	if (pos < 0)
+		return 0;
+	return Util_stricmp(s1 + pos, s2) == 0;
+}
+
 int Util_strnicmp(const char *str1, const char *str2, size_t size)
 {
 	int retval = 0;
@@ -352,6 +368,74 @@ void Util_catpath(char *result, const char *path1, const char *path2)
 			? "%s%s" : "%s" Util_DIR_SEP_STR "%s", path1, path2);
 }
 
+static int parse_hashes(const char *p, char *buffer, int bufsize)
+{
+	char *f = buffer;
+	char no_width = '0';
+	int no_max = 1;
+	/* 9 because sprintf'ed "no" can be 9 digits */
+	while (f < buffer + bufsize - 9) {
+		/* replace a sequence of hashes with e.g. "%05d" */
+		if (*p == '#') {
+			if (no_width > '0') /* already seen a sequence of hashes */
+				break;          /* invalid */
+			/* count hashes */
+			do {
+				no_max *= 10;
+				p++;
+				no_width++;
+				/* now no_width is the number of hashes seen so far
+				   and p points after the counted hashes */
+			} while (no_width < '9' && *p == '#'); /* no more than 9 hashes */
+			*f++ = '%';
+			*f++ = '0';
+			*f++ = no_width;
+			*f++ = 'd';
+			continue;
+		}
+		if (*p == '%')
+			*f++ = '%'; /* double the percents */
+		*f++ = *p;
+		if (*p == '\0')
+			return no_max; /* ok */
+		p++;
+	}
+	return 0;
+}
+
+int Util_filenamepattern(const char *p, char *buffer, int bufsize, const char *default_pattern)
+{
+	int no_max;
+
+	no_max = parse_hashes(p, buffer, bufsize);
+	if (!no_max && default_pattern) {
+		Log_print("Invalid filename pattern, using default.");
+		no_max = parse_hashes(default_pattern, buffer, bufsize);
+	}
+	return no_max;
+}
+
+int Util_findnextfilename(const char *format, int *no_last, int no_max, char *buffer, int bufsize, int allow_overwrite)
+{
+	int no;
+
+	/* negative number to initialize */
+	if (*no_last < 0) *no_last = -1;
+	no = *no_last;
+	for (;;) {
+		if ((++no >= no_max) & !allow_overwrite) {
+			return FALSE;
+		}
+		snprintf(buffer, bufsize, format, no % no_max);
+		*no_last = no;
+		if ((no >= no_max) && allow_overwrite)
+			break;
+		if (!Util_fileexists(buffer))
+			break; /* file does not exist - we can create it */
+	}
+	return TRUE;
+}
+
 int Util_fileexists(const char *filename)
 {
 	FILE *fp;
@@ -532,3 +616,17 @@ void Util_sleep(double s)
 	}
 #endif /* !SUPPORTS_PLATFORM_SLEEP */
 }
+
+char *Util_getcwd(char *buf, size_t size)
+{
+#ifdef HAVE_GETCWD
+	if (getcwd(buf, size) == NULL) {
+		buf[0] = '.';
+		buf[1] = '\0';
+	}
+#else
+	buf[0] = '.';
+	buf[1] = '\0';
+#endif
+	return buf;
+}
